/**
 * prog1.c
 *
 * Kaynak:
 * Murat Demirten,
 * Linux Sistem Programlama kitabından uyarlanmıştır.
 *
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <sys/time.h>
#include <errno.h>
#include <string.h>

void yakala(int sig){
   write(STDOUT_FILENO, "sinyal yedim.\n", 14);
   exit(EXIT_FAILURE);
}

void timer_callback (int signum, siginfo_t *si, void *context){

    char   *p;
    struct  timeval now;

    gettimeofday(&now, NULL);

    p= (char *)si->si_value.sival_ptr;

    fprintf(stdout, "signal %d caught on %ld.%03ld\t name: %s\n",
                    signum, now.tv_sec, now.tv_usec/1000, p);
}

int main (int argc, char *argv[]){

    char *msg1= "birinci zamanlayıcı";
    char *msg2= "ikinci zamanlayıcı";

    timer_t timer1;
    timer_t timer2;

    struct itimerspec new_value;

    struct sigaction sa;
    struct sigevent  sev;
    int    ret;

    sa.sa_flags = SA_SIGINFO;     // 3 argümanlı call back için zorunludur.
    sa.sa_sigaction = timer_callback;
    sigemptyset(&sa.sa_mask);

    if (sigaction(SIGRTMIN, &sa, NULL) == -1) {
       perror("sigaction()");
       return EXIT_FAILURE;
       }

    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_signo  = SIGRTMIN;

    // Birinci zamanlayıcı.
    sev.sigev_value.sival_ptr= msg1;

    if ( (ret = timer_create(CLOCK_MONOTONIC, &sev, &timer1)) < 0) {
       perror("timer_create()");
       return EXIT_FAILURE;
       }

    // İkinci zamanlayıcı.
    sev.sigev_value.sival_ptr = msg2;

    if ( (ret = timer_create(CLOCK_MONOTONIC, &sev, &timer2)) < 0) {
       perror("timer_create()");
       return EXIT_FAILURE;
       }

    // Bekleme süresi.
    new_value.it_value.tv_sec     = 2;
    new_value.it_value.tv_nsec    = 0;

    // Periyot.
    // 1,000,000,000 nsec = 1sec
    new_value.it_interval.tv_sec  = 0;
    new_value.it_interval.tv_nsec = 500 * 1000 * 1000;

    // 1. zamanlayıcıyı başlat.
    //
    // flag= 0: new_value değeri, timer_settime() komutu girildikten sonraki zamandan sayar.
    //
    // flag= TIMER_ABSTIME durumunda ise new_value değeri, gerçek zaman olarak kabul edilir.
    //       Bu zaman geçilmişse hemen periyot başlar.
    //       Bakınız, gettimeofday();
    //
    if (timer_settime(timer1, 0, &new_value, NULL) == -1 ){
       perror("timer_settime()");
       return EXIT_FAILURE;
       }
      
    new_value.it_interval.tv_nsec = 250 * 1000 * 1000;

    // 2. zamanlayıcıyı başlat.
    if ( timer_settime(timer2, 0, &new_value, NULL) == -1 ){
       perror("timer_settime()");
       return EXIT_FAILURE;
       }

    signal(SIGINT, yakala);
      
    // gethcar() sinyal yediği için hemen döner.
    // Interrupted system call 
    // Bu durumda errno değeri EINTR olur.
    // getchar() sinyal yediğinde EOF döner.
    //
    while( getchar() ){
         // if ( errno != EINTR ) break;
         // fprintf(stdout, "bekliyorum...\n");
         }

    // Yürütme buraya gelmez.
    //
    // timer_gettime() ile de mevcut bir timer'da 
    // kalan süre elde edilebilir.

    timer_delete(timer1);
    timer_delete(timer2);

    return EXIT_SUCCESS;
}

