/* prog4.c 
 *
 * getaddrinfo() örneği
 *
 * IPv4/IPv6, isim/adres/port çözümlemesi yapar.
 *
 * Örnek
 * $ ./prog3 google.com
 * host: google.com
 * IPv4 address: 216.58.206.174 (google.com)
 * IPv6 address: 2a00:1450:4017:806::200e ((null))
 *
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

int lookup_host(char *host){

  struct addrinfo hints, *res;
  int    err;
  char   addr_str[128];
  void  *ptr;

  memset (&hints, 0, sizeof (hints));

  hints.ai_family  = PF_UNSPEC;    // v4 veya v6 olabilir.
  hints.ai_socktype= SOCK_STREAM; 
  hints.ai_flags   = AI_CANONNAME;

  err= getaddrinfo (host, NULL, &hints, &res);

  if (err){
     fprintf(stderr, "%s: %s.\n", host, gai_strerror(err)); 
     return -1;
     }

  fprintf(stdout, "host: %s\n", host);

  while(res){

       inet_ntop(res->ai_family, res->ai_addr->sa_data, addr_str, sizeof(addr_str));

       if (res->ai_family== AF_INET ){
          ptr= &((struct sockaddr_in *) res->ai_addr)->sin_addr;
          }
          else{
          ptr= &((struct sockaddr_in6 *) res->ai_addr)->sin6_addr;
          }

      inet_ntop(res->ai_family, ptr, addr_str, 100);
      printf ("IPv%d address: %s (%s)\n", res->ai_family== PF_INET6 ? 6 : 4,
                                          addr_str, res->ai_canonname);
      res = res->ai_next;

      } // while

  return 0;
}

int main (int argc, char *argv[]){

  if (argc != 2){
     fprintf(stderr, "kullanımı: %s hostName\n", argv[0]);
     return EXIT_FAILURE;
     }

  return lookup_host(argv[1])==0 ? EXIT_SUCCESS:EXIT_FAILURE;
}

