/**
 * prog2.c
 *
 * Process based tcp/ip server.
 *
 */
 
#include <stdio.h>
#include <stdlib.h>  
#include <string.h>  
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <arpa/inet.h> 
#include <ctype.h>   
#include <unistd.h>   
#include <pthread.h>   
#include <stdint.h>   

/**
 * İstemciden gelenleri ekrana yaz.
 * Aynı zamanda gelen veriyi büyüt ve geri gönder.
 *
 */
void client_task(int fd){

   char      buf[256];
   int       n;
   char     *p;

   fprintf(stdout, "çocuk başladı fd:%d pid:%d\n", fd, getpid());

   for(;;){
  
      n= read(fd, buf, sizeof(buf)- 1);
  
      if ( n == 0 ){
         fprintf(stdout, "fd=%d, bağlantı kesildi.\n", fd);
           break;
         }
  
      if ( n < 0 ){
         perror("read()");
           break;
         }
  
      buf[n]= '\0';
      fprintf(stdout, "%d --> %s", fd, buf);

      // Gelen veriyi büyüterek geri gönder.
      for(p= buf; *p != '\0'; p++){
         *p= toupper(*p);
         }

      if ( write(fd, buf, n) == -1 ){   // hata?
         perror("write()");
         break;
         }
  
      } // for ;;

   if ( close(fd) == -1 ){
      perror("close()");
      }

   fprintf(stdout, "çocuk bitti fd:%d pid:%d\n", fd, getpid());

   exit(EXIT_SUCCESS);
}
 
int main(int argc , char *argv[]){

   int  server_fd;
   int  client_fd;
   int  addr_len;
   int  port;

   struct sockaddr_in client;
   struct sockaddr_in server;

   if ( argc != 2 ){
      fprintf(stderr, "kullanımı: %s port\n", argv[0]);
      return EXIT_FAILURE;
      }

   // Soket kur.
   server_fd= socket(AF_INET , SOCK_STREAM , 0);

   if (server_fd == -1){
      perror("socket()");
      return EXIT_FAILURE;
      }
   
   // Sunucu durup tekrar başladığında
   // Address already in use hatasını engellemek içindir.
   int optval = 1;
   setsockopt(server_fd, SOL_SOCKET, SO_REUSEADDR, (const void *)&optval , sizeof(int));

   port= atoi(argv[1]);

   // Nereden dinleyecek?
   server.sin_family     = AF_INET;
   server.sin_addr.s_addr= INADDR_ANY; // 0.0.0.0
   server.sin_port       = htons(port);
    
   if ( bind(server_fd, (struct sockaddr *)&server , sizeof(server)) < 0){
      perror("bind failed. Error");
      return EXIT_FAILURE;
      }
    
   // Pasif hale getir.
   listen(server_fd, 5);
   
   for(;;){

      // Bağlantı bekle.
      fprintf(stdout, "bağlantı bekliyorum, port:%d  pid:%d\n", port, getpid());
  
      addr_len= sizeof(struct sockaddr_in);
       
      // connect() gelene kadar blokla.
      client_fd= accept(server_fd, (struct sockaddr *)&client, (socklen_t*)&addr_len);
  
      if (client_fd < 0){
         perror("accept failed");
         return EXIT_FAILURE;
         }
  
      fprintf(stdout, "bağlantı kabul edildi, %s:%d\n", 
                      inet_ntoa(client.sin_addr), ntohs(client.sin_port));
       
      // Her bir bağlantı için yeni bir proses yarat.

      if ( fork() == 0 ){
         // Çocuktayım.
         close(server_fd); 
         client_task(client_fd);
         }

      // Annedeyim.
      close(client_fd);

      } // for ;;
     
    fprintf(stdout, "program bitti.\n");

    return EXIT_SUCCESS;
}

