/**
 * prog2.c
 *
 * i/o multiplexing.
 *
 */
 
#include <stdio.h>
#include <stdlib.h>  
#include <string.h>  
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <arpa/inet.h> 
#include <ctype.h>   
#include <unistd.h>   
#include <pthread.h>   
#include <stdint.h>   

/**
 * İstemciden gelenleri ekrana yazar.
 * Aynı zamanda gelen veriyi büyüt ve geri gönderir.
 *
 * Hata yoksa 0, varsa <0 döner.
 *
 */
int client_task(int fd){

   char    buf[256];
   int     n;
   char   *p;

   n= read(fd, buf, sizeof(buf)- 1);
  
   if ( n == 0 ){
      fprintf(stdout, "fd=%d, bağlantı kesildi.\n", fd);
      return -1;
      }
  
   if ( n < 0 ){
      perror("read()");
      return -1;
      }
  
   buf[n]= '\0';
   fprintf(stdout, "%d --> %s", fd, buf);

   // Gelen veriyi büyüterek geri gönder.
   for(p= buf; *p != '\0'; p++){
      *p= toupper(*p);
      }

   if ( write(fd, buf, n) == -1 ){   // hata?
      perror("write()");
      return -1;
      }
  
   return 0;
}
 
int find_max_fd(int max_fd, fd_set *readfs){

   int i;

   for(i= max_fd- 1; i >= 0; i--){

      if ( FD_ISSET(i, readfs) ){
         return i;
         }

      } 

   // En az 1 fd olmalı!
   fprintf(stderr, "%s:%d: BÖCEK\n", __FILE__, __LINE__);
   exit(EXIT_FAILURE);
}

int main(int argc , char *argv[]){

   int  server_fd;
   int  client_fd;
   int  addr_len;
   int  port;

   struct sockaddr_in client;
   struct sockaddr_in server;

   if ( argc != 2 ){
      fprintf(stderr, "kullanımı: %s port\n", argv[0]);
      return EXIT_FAILURE;
      }

   // Soket kur.
   server_fd= socket(AF_INET , SOCK_STREAM , 0);

   if (server_fd == -1){
      perror("socket()");
      return EXIT_FAILURE;
      }
   
   // Sunucu durup tekrar başladığında
   // Address already in use hatasını engellemek içindir.
   int optval = 1;
   setsockopt(server_fd, SOL_SOCKET, SO_REUSEADDR, (const void *)&optval , sizeof(int));

   port= atoi(argv[1]);

   // Nereden dinleyecek?
   server.sin_family     = AF_INET;
   server.sin_addr.s_addr= INADDR_ANY; // 0.0.0.0
   server.sin_port       = htons(port);
    
   if ( bind(server_fd, (struct sockaddr *)&server , sizeof(server)) < 0){
      perror("bind failed. Error");
      return EXIT_FAILURE;
      }
    
   // Pasif hale getir.
   listen(server_fd, 5);

   // senkron i/o multiplexing
   //
   fd_set     tmpfs;
   fd_set     readfs;
   int        result;
   int        max_fd;
   int        i;

   struct timeval t;

   FD_ZERO(&readfs);
   FD_SET(server_fd, &readfs);
   max_fd= server_fd;

   for(;;){
      tmpfs= readfs;

      t.tv_sec = 5;
      t.tv_usec= 0;

      fprintf(stdout, "bağlantı/veri bekliyorum, port:%d  max_fd:%d\n", port, max_fd);
      result= select(max_fd+ 1, &tmpfs, 0, 0, &t);

      if ( result == 0 ){
         fprintf(stdout, "süre aşımı.\n"); 
         continue;
         }

      if ( result < 0 ){
         perror("select()");
         break;
         }

      // Bağlantı talebi gelmiş.
      //
      if ( FD_ISSET(server_fd, &tmpfs) ){

         addr_len= sizeof(struct sockaddr_in);
         client_fd= accept(server_fd, (struct sockaddr *)&client, (socklen_t*)&addr_len);
  
         if (client_fd < 0){
            perror("accept()");
            break;
            }
  
         fprintf(stdout, "bağlantı kabul edildi, %s:%d\n", 
                         inet_ntoa(client.sin_addr), ntohs(client.sin_port));

         FD_SET(client_fd, &readfs);
         if ( client_fd > max_fd) max_fd= client_fd;

         result--;

         } // if ( FD_ISSET()) 


      // Veri gelmiş veya client kapanmış.
      // 
      for(i= 0; result > 0 && i<= max_fd; i++){

         if ( ! FD_ISSET(i, &tmpfs) ){
            continue;
            } 

         if ( i== server_fd ){
            // yukarıda işlendi, bir daha işlemeye gerek yok.
            continue;
            }

         result--;

         if ( client_task(i) < 0 ){
            shutdown(i, SHUT_RDWR);
            close(i);
            FD_CLR(i, &readfs);
            if ( i== max_fd ) max_fd= find_max_fd(max_fd, &readfs);
            }

         }  /* for */

      } // for(;;) 

   // Hata dışında yürütme buraya gelmez.
   return EXIT_SUCCESS;
}


